@@ -40,7 +40,7 @@ public:
         m_encDefault = enc;
     }
 
-    // copy ctor doesn't initialize anything neither as conversion can only be
+    // copy ctor doesn't initialize anything either as conversion can only be
     // deduced on first use
     wxConvAuto(const wxConvAuto& other) : wxMBConv()
     {
@@ -71,14 +71,16 @@ public:
 
     // override the base class virtual function(s) to use our m_conv
     virtual size_t ToWChar(wchar_t *dst, size_t dstLen,
-                           const char *src, size_t srcLen = wxNO_LEN) const wxOVERRIDE;
+                           const char *src, size_t srcLen = wxNO_LEN) const override;
 
     virtual size_t FromWChar(char *dst, size_t dstLen,
-                             const wchar_t *src, size_t srcLen = wxNO_LEN) const wxOVERRIDE;
+                             const wchar_t *src, size_t srcLen = wxNO_LEN) const override;
 
-    virtual size_t GetMBNulLen() const wxOVERRIDE { return m_conv->GetMBNulLen(); }
+    virtual size_t GetMBNulLen() const override { return m_conv->GetMBNulLen(); }
 
-    virtual wxMBConv *Clone() const wxOVERRIDE { return new wxConvAuto(*this); }
+    virtual bool IsUTF8() const override { return m_conv && m_conv->IsUTF8(); }
+
+    wxNODISCARD virtual wxMBConv *Clone() const override { return new wxConvAuto(*this); }
 
     // return the BOM type of this buffer
     static wxBOM DetectBOM(const char *src, size_t srcLen);
@@ -91,13 +93,21 @@ public:
         return m_bomType;
     }
 
+    wxFontEncoding GetEncoding() const;
+
+    // Return true if the fall-back encoding is used
+    bool IsUsingFallbackEncoding() const
+    {
+        return m_ownsConv && m_bomType == wxBOM_None;
+    }
+
 private:
     // common part of all ctors
     void Init()
     {
         // We don't initialize m_encDefault here as different ctors do it
         // differently.
-        m_conv = NULL;
+        m_conv = nullptr;
         m_bomType = wxBOM_Unknown;
         m_ownsConv = false;
         m_consumedBOM = false;
@@ -129,7 +139,7 @@ private:
     // wxFONTENCODING_MAX but not wxFONTENCODING_DEFAULT
     static wxFontEncoding ms_defaultMBEncoding;
 
-    // conversion object which we really use, NULL until the first call to
+    // conversion object which we really use, null until the first call to
     // either ToWChar() or FromWChar()
     wxMBConv *m_conv;
 
