@@ -2,7 +2,6 @@
 // Name:        wx/univ/combobox.h
 // Purpose:     the universal combobox
 // Author:      Vadim Zeitlin
-// Modified by:
 // Created:     30.08.00
 // Copyright:   (c) 2000 SciTech Software, Inc. (www.scitechsoft.com)
 // Licence:     wxWindows licence
@@ -46,10 +45,10 @@ public:
                const wxPoint& pos = wxDefaultPosition,
                const wxSize& size = wxDefaultSize,
                int n = 0,
-               const wxString choices[] = (const wxString *) NULL,
+               const wxString choices[] = nullptr,
                long style = 0,
                const wxValidator& validator = wxDefaultValidator,
-               const wxString& name = wxComboBoxNameStr)
+               const wxString& name = wxASCII_STR(wxComboBoxNameStr))
     {
         Init();
 
@@ -64,7 +63,7 @@ public:
                const wxArrayString& choices,
                long style = 0,
                const wxValidator& validator = wxDefaultValidator,
-               const wxString& name = wxComboBoxNameStr);
+               const wxString& name = wxASCII_STR(wxComboBoxNameStr));
 
     bool Create(wxWindow *parent,
                 wxWindowID id,
@@ -72,10 +71,10 @@ public:
                 const wxPoint& pos = wxDefaultPosition,
                 const wxSize& size = wxDefaultSize,
                 int n = 0,
-                const wxString choices[] = (const wxString *) NULL,
+                const wxString choices[] = nullptr,
                 long style = 0,
                 const wxValidator& validator = wxDefaultValidator,
-                const wxString& name = wxComboBoxNameStr);
+                const wxString& name = wxASCII_STR(wxComboBoxNameStr));
     bool Create(wxWindow *parent,
                 wxWindowID id,
                 const wxString& value,
@@ -84,7 +83,7 @@ public:
                 const wxArrayString& choices,
                 long style = 0,
                 const wxValidator& validator = wxDefaultValidator,
-                const wxString& name = wxComboBoxNameStr);
+                const wxString& name = wxASCII_STR(wxComboBoxNameStr));
 
     virtual ~wxComboBox();
 
@@ -94,35 +93,35 @@ public:
     // implement the combobox interface
 
     // wxTextCtrl methods
-    virtual wxString GetValue() const wxOVERRIDE { return DoGetValue(); }
-    virtual void SetValue(const wxString& value) wxOVERRIDE;
-    virtual void WriteText(const wxString& value) wxOVERRIDE;
-    virtual void Copy() wxOVERRIDE;
-    virtual void Cut() wxOVERRIDE;
-    virtual void Paste() wxOVERRIDE;
-    virtual void SetInsertionPoint(long pos) wxOVERRIDE;
-    virtual void SetInsertionPointEnd() wxOVERRIDE;
-    virtual long GetInsertionPoint() const wxOVERRIDE;
-    virtual wxTextPos GetLastPosition() const wxOVERRIDE;
-    virtual void Replace(long from, long to, const wxString& value) wxOVERRIDE;
-    virtual void Remove(long from, long to) wxOVERRIDE;
-    virtual void SetSelection(long from, long to) wxOVERRIDE;
-    virtual void GetSelection(long *from, long *to) const wxOVERRIDE;
-    virtual void SetEditable(bool editable) wxOVERRIDE;
-    virtual bool IsEditable() const wxOVERRIDE;
+    virtual wxString GetValue() const override { return DoGetValue(); }
+    virtual void SetValue(const wxString& value) override;
+    virtual void WriteText(const wxString& value) override;
+    virtual void Copy() override;
+    virtual void Cut() override;
+    virtual void Paste() override;
+    virtual void SetInsertionPoint(long pos) override;
+    virtual void SetInsertionPointEnd() override;
+    virtual long GetInsertionPoint() const override;
+    virtual wxTextPos GetLastPosition() const override;
+    virtual void Replace(long from, long to, const wxString& value) override;
+    virtual void Remove(long from, long to) override;
+    virtual void SetSelection(long from, long to) override;
+    virtual void GetSelection(long *from, long *to) const override;
+    virtual void SetEditable(bool editable) override;
+    virtual bool IsEditable() const override;
 
-    virtual void Undo() wxOVERRIDE;
-    virtual void Redo() wxOVERRIDE;
-    virtual void SelectAll() wxOVERRIDE;
+    virtual void Undo() override;
+    virtual void Redo() override;
+    virtual void SelectAll() override;
 
-    virtual bool CanCopy() const wxOVERRIDE;
-    virtual bool CanCut() const wxOVERRIDE;
-    virtual bool CanPaste() const wxOVERRIDE;
-    virtual bool CanUndo() const wxOVERRIDE;
-    virtual bool CanRedo() const wxOVERRIDE;
+    virtual bool CanCopy() const override;
+    virtual bool CanCut() const override;
+    virtual bool CanPaste() const override;
+    virtual bool CanUndo() const override;
+    virtual bool CanRedo() const override;
 
     // override these methods to disambiguate between two base classes versions
-    virtual void Clear() wxOVERRIDE
+    virtual void Clear() override
     {
         wxItemContainer::Clear();
     }
@@ -132,15 +131,15 @@ public:
     bool IsTextEmpty() const { return wxTextEntry::IsEmpty(); }
 
     // wxControlWithItems methods
-    virtual void DoClear() wxOVERRIDE;
-    virtual void DoDeleteOneItem(unsigned int n) wxOVERRIDE;
-    virtual unsigned int GetCount() const wxOVERRIDE;
-    virtual wxString GetString(unsigned int n) const wxOVERRIDE;
-    virtual void SetString(unsigned int n, const wxString& s) wxOVERRIDE;
-    virtual int FindString(const wxString& s, bool bCase = false) const wxOVERRIDE;
-    virtual void SetSelection(int n) wxOVERRIDE;
-    virtual int GetSelection() const wxOVERRIDE;
-    virtual wxString GetStringSelection() const wxOVERRIDE;
+    virtual void DoClear() override;
+    virtual void DoDeleteOneItem(unsigned int n) override;
+    virtual unsigned int GetCount() const override;
+    virtual wxString GetString(unsigned int n) const override;
+    virtual void SetString(unsigned int n, const wxString& s) override;
+    virtual int FindString(const wxString& s, bool bCase = false) const override;
+    virtual void SetSelection(int n) override;
+    virtual int GetSelection() const override;
+    virtual wxString GetStringSelection() const override;
 
     // we have our own input handler and our own actions
     // (but wxComboCtrl already handled Popup/Dismiss)
@@ -151,23 +150,25 @@ public:
     */
 
     static wxInputHandler *GetStdInputHandler(wxInputHandler *handlerDef);
-    virtual wxInputHandler *DoGetStdInputHandler(wxInputHandler *handlerDef) wxOVERRIDE
+    virtual wxInputHandler *DoGetStdInputHandler(wxInputHandler *handlerDef) override
     {
         return GetStdInputHandler(handlerDef);
     }
 
     // we delegate our client data handling to wxListBox which we use for the
     // items, so override this and other methods dealing with the client data
-    virtual wxClientDataType GetClientDataType() const wxOVERRIDE;
-    virtual void SetClientDataType(wxClientDataType clientDataItemsType) wxOVERRIDE;
+    virtual wxClientDataType GetClientDataType() const override;
+    virtual void SetClientDataType(wxClientDataType clientDataItemsType) override;
+
+    virtual const wxTextEntry* WXGetTextEntry() const override { return this; }
 
 protected:
     virtual int DoInsertItems(const wxArrayStringsAdapter& items,
                               unsigned int pos,
-                              void **clientData, wxClientDataType type) wxOVERRIDE;
+                              void **clientData, wxClientDataType type) override;
 
-    virtual void DoSetItemClientData(unsigned int n, void* clientData) wxOVERRIDE;
-    virtual void* DoGetItemClientData(unsigned int n) const wxOVERRIDE;
+    virtual void DoSetItemClientData(unsigned int n, void* clientData) override;
+    virtual void* DoGetItemClientData(unsigned int n) const override;
 
 
     // common part of all ctors
@@ -178,7 +179,7 @@ protected:
 
 private:
     // implement wxTextEntry pure virtual method
-    virtual wxWindow *GetEditableWindow() wxOVERRIDE { return this; }
+    virtual wxWindow *GetEditableWindow() override { return this; }
 
     // the popup listbox
     wxListBox *m_lbox;
