@@ -9,16 +9,15 @@
 
 /**
     Flags for wxColour -> wxString conversion (see wxColour::GetAsString).
-
-    @{
 */
+///@{
 enum {
     wxC2S_NAME             = 1,   // return colour name, when possible
     wxC2S_CSS_SYNTAX       = 2,   // return colour in rgb(r,g,b) syntax
     wxC2S_HTML_SYNTAX      = 4    // return colour in #rrggbb syntax
 };
 
-//@}
+///@}
 
 const unsigned char wxALPHA_TRANSPARENT = 0;
 const unsigned char wxALPHA_OPAQUE = 0xff;
@@ -27,14 +26,28 @@ const unsigned char wxALPHA_OPAQUE = 0xff;
     @class wxColour
 
     A colour is an object representing a combination of Red, Green, and Blue
-    (RGB) intensity values, and is used to determine drawing colours. See the
-    entry for wxColourDatabase for how a pointer to a predefined, named colour
-    may be returned instead of creating a new colour.
+    (RGB) intensity values and an Alpha value, and is used to determine
+    drawing colours. See the entry for wxColourDatabase for how a pointer to a predefined,
+    named colour may be returned instead of creating a new colour.
 
     Valid RGB values are in the range 0 to 255.
 
     You can retrieve the current system colour settings with wxSystemSettings.
 
+
+    @section colour_accessors Channel Accessor Functions
+
+    Note that this class provides pairs of functions for each of the colour
+    channels, i.e. red, green, blue and alpha values. The one word functions
+    Red(), Green(), Blue() and Alpha() return the values of type @c unsigned @c
+    char, while GetRed(), GetGreen(), GetBlue() and GetAlpha() returns the same
+    value as @c unsigned @c int. According to the C++ integer promotion rules,
+    the result of any arithmetic expression involving the former will be
+    (signed) @c int, while that of the latter will be @c unsigned, which is
+    what would be commonly expected, so the latter family of functions should
+    be typically preferred (but they are only available since wxWidgets 3.1.6).
+
+
     @library{wxcore}
     @category{gdi}
 
@@ -86,7 +99,7 @@ public:
             A packed RGB value.
     */
     wxColour(unsigned long colRGB);
-    
+
     /**
         Copy constructor.
     */
@@ -95,14 +108,47 @@ public:
     /**
         Returns the alpha value, on platforms where alpha is not yet supported, this
         always returns wxALPHA_OPAQUE.
+
+        @see GetAlpha()
     */
     virtual unsigned char Alpha() const;
 
     /**
         Returns the blue intensity.
+
+        @see GetBlue()
     */
     virtual unsigned char Blue() const;
 
+    /**
+        Returns the alpha value, on platforms where alpha is not yet supported, this
+        always returns wxALPHA_OPAQUE.
+
+        @since 3.1.6
+     */
+    unsigned int GetAlpha() const;
+
+    /**
+        Returns the blue intensity as unsigned int.
+
+        @since 3.1.6
+     */
+    unsigned int GetBlue() const;
+
+    /**
+        Returns the green intensity as unsigned int.
+
+        @since 3.1.6
+     */
+    unsigned int GetGreen() const;
+
+    /**
+        Returns the red intensity as unsigned int.
+
+        @since 3.1.6
+     */
+    unsigned int GetRed() const;
+
     /**
         Converts this colour to a wxString using the given flags.
 
@@ -113,22 +159,26 @@ public:
         @c wxC2S_HTML_SYNTAX, to obtain the colour as "#" followed by 6
         hexadecimal digits (e.g. wxColour(255,0,0) == "#FF0000").
 
-        This function never fails and always returns a non-empty string but
-        asserts if the colour has alpha channel (i.e. is non opaque) but
-        @c wxC2S_CSS_SYNTAX (which is the only one supporting alpha) is not
-        specified in flags.
+        This function returns empty string if the colour is not initialized
+        (see IsOk()). Otherwise, the returned string is always non-empty, but
+        the function asserts if the colour has alpha channel (i.e. is non
+        opaque) but @c wxC2S_CSS_SYNTAX (which is the only one supporting
+        alpha) is not specified in @a flags.
+
+        @note For non-solid (i.e. non-RGB) colour this function returns
+        "rgb(??, ?? ??)" or "#??????".
 
         @since 2.7.0
     */
     virtual wxString GetAsString(long flags = wxC2S_NAME | wxC2S_CSS_SYNTAX) const;
 
-    //@{
+    ///@{
     /**
         Sets the RGB or RGBA colour values from a single 32 bit value.
 
         The arguments @a colRGB and @a colRGBA should be of the form 0x00BBGGRR
         and 0xAABBGGRR respectively where @c 0xRR, @c 0xGG, @c 0xBB and @c 0xAA
-        are the values of the red, blue, green and alpha components.
+        are the values of the red, green, blue and alpha components.
 
         Notice the right-to-left order of components!
 
@@ -138,9 +188,9 @@ public:
     */
     void SetRGB(wxUint32 colRGB);
     void SetRGBA(wxUint32 colRGBA);
-    //@}
+    ///@}
 
-    //@{
+    ///@{
     /**
         Gets the RGB or RGBA colour values as a single 32 bit value.
 
@@ -156,7 +206,20 @@ public:
     */
     wxUint32 GetRGB() const;
     wxUint32 GetRGBA() const;
-    //@}
+    ///@}
+
+    /**
+        Return the perceived brightness of the colour.
+
+        This value is computed using the simple @code 0.299*R + 0.587*G + 0.114*B @endcode
+        formula with the coefficients taken from the RGB to
+        YIQ conversion formula and @c R, @c G and @c B being the values of the
+        corresponding colour channels normalized to 0..1 range, so that the
+        return value is 0 for black and 1 for white.
+
+        @since 3.1.3
+     */
+    double GetLuminance() const;
 
     /**
         Returns a pixel value which is platform-dependent.
@@ -168,6 +231,8 @@ public:
 
     /**
         Returns the green intensity.
+
+        @see GetGreen()
     */
     virtual unsigned char Green() const;
 
@@ -179,23 +244,50 @@ public:
 
     /**
         Returns the red intensity.
+
+        @see GetRed()
     */
     virtual unsigned char Red() const;
 
     /**
-        Returns @true if the color can be described using RGB values, ie is solid,
+        Returns @true if the color can be described using RGB values, i.e. is solid,
         @false if it is a pattern (currently only possible on macOS)
+
+        @since 3.1.2
     */
     virtual bool IsSolid() const;
-    //@{
+
+    /**
+        Returns @true if the color is completely transparent (i.e., no opacity).
+
+        @since 3.3.1
+    */
+    bool IsTransparent() const;
+
+    /**
+        Returns @true if the color is completely opaque (i.e., full opacity).
+
+        @since 3.3.1
+    */
+    bool IsOpaque() const;
+
+    /**
+        Returns @true if the color has some translucency
+        (not fully opaque, but not transparent either).
+
+        @since 3.3.1
+    */
+    bool IsTranslucent() const;
+
+    ///@{
     /**
         Sets the RGB intensity values using the given values (first overload),
         extracting them from the packed long (second overload), using the given
         string (third overload).
 
         When using third form, Set() accepts: colour names (those listed in
-        wxColourDatabase), the CSS-like @c "rgb(r,g,b)" or @c "rgba(r,g,b,a)" syntax 
-        (case insensitive) and the HTML-like syntax: @c "#" followed by 6 hexadecimal 
+        wxColourDatabase), the CSS-like @c "rgb(r,g,b)" or @c "rgba(r,g,b,a)" syntax
+        (case insensitive) and the HTML-like syntax: @c "#" followed by 6 hexadecimal
         digits for red, green, blue components.
 
         Returns @true if the conversion was successful, @false otherwise.
@@ -207,16 +299,16 @@ public:
              unsigned char alpha = wxALPHA_OPAQUE);
     void Set(unsigned long RGB);
     bool Set(const wxString& str);
-    //@}
+    ///@}
 
     /**
         Tests the inequality of two colours by comparing individual red, green, blue
-        colours and alpha values.
+        intensities and alpha values.
     */
     bool operator !=(const wxColour& colour) const;
 
     /**
-        Assignment operator, using a colour name to be found in the colour database.
+        Assignment operator.
 
         @see wxColourDatabase
     */
@@ -224,16 +316,16 @@ public:
 
     /**
         Tests the equality of two colours by comparing individual red, green, blue
-        colours and alpha values.
+        intensities and alpha values.
     */
     bool operator ==(const wxColour& colour) const;
 
     /**
-        Assign 0 or 255 to rgb out parameters.
+        Assigns the same value to @a r, @a g, @a b: 0 if @a on is @c false, 255 otherwise.
         @since 2.9.0
     */
     static void MakeMono(unsigned char* r, unsigned char* g, unsigned char* b, bool on);
-    
+
     /**
         Create a disabled (dimmed) colour from (in/out) rgb parameters.
         @since 2.9.0
@@ -254,7 +346,7 @@ public:
         @since 2.9.0
     */
     static void MakeGrey(unsigned char* r, unsigned char* g, unsigned char* b);
-    
+
     /**
         Create a grey colour from (in/out) rgb parameters using floating point arithmetic.
         Defaults to using the standard ITU-T BT.601 when converting to YUV, where every pixel equals
@@ -269,12 +361,11 @@ public:
         @since 2.9.0
     */
     static unsigned char AlphaBlend(unsigned char fg, unsigned char bg, double alpha);
-    
+
     /**
-        ChangeLightness() is a utility function that simply darkens
-        or lightens a color, based on the specified percentage
-        ialpha of 0 would be completely black, 200 completely white
-        an ialpha of 100 returns the same colour
+        Utility function that simply darkens or lightens a color, based on the specified
+        percentage @a ialpha. @a ialpha of 0 would be make the color completely black,
+        200 completely white and 100 would not change the color.
         @since 2.9.0
     */
     static void ChangeLightness(unsigned char* r, unsigned char* g, unsigned char* b, int ialpha);
@@ -288,7 +379,7 @@ public:
 
 
 /** @name Predefined colors. */
-//@{
+///@{
 wxColour wxNullColour;
 wxColour wxTransparentColour;
 wxColour* wxBLACK;
@@ -299,7 +390,7 @@ wxColour* wxYELLOW;
 wxColour* wxLIGHT_GREY;
 wxColour* wxRED;
 wxColour* wxWHITE;
-//@}
+///@}
 
 
 
@@ -308,7 +399,7 @@ wxColour* wxWHITE;
 // ============================================================================
 
 /** @addtogroup group_funcmacro_misc */
-//@{
+///@{
 
 /**
     Converts string to a wxColour best represented by the given string. Returns
@@ -329,5 +420,5 @@ bool wxFromString(const wxString& string, wxColour* colour);
 */
 wxString wxToString(const wxColour& colour);
 
-//@}
+///@}
 
