@@ -204,12 +204,12 @@ public:
     virtual int DrawHeaderButton(wxWindow *win, wxDC& dc,
                                  const wxRect& rect, int flags = 0,
                                  wxHeaderSortIconType sortArrow = wxHDR_SORT_ICON_NONE,
-                                 wxHeaderButtonParams* params = NULL);
+                                 wxHeaderButtonParams* params = nullptr);
 
     virtual int DrawHeaderButtonContents(wxWindow *win, wxDC& dc,
                                          const wxRect& rect, int flags = 0,
                                          wxHeaderSortIconType sortArrow = wxHDR_SORT_ICON_NONE,
-                                         wxHeaderButtonParams* params = NULL);
+                                         wxHeaderButtonParams* params = nullptr);
 
     virtual int GetHeaderButtonHeight(wxWindow *win);
 
@@ -234,7 +234,14 @@ public:
     virtual void DrawCheckBox(wxWindow *win, wxDC& dc,
                               const wxRect& rect, int flags = 0 );
 
-    virtual wxSize GetCheckBoxSize(wxWindow *win);
+    virtual void DrawCheckMark(wxWindow *win, wxDC& dc,
+                               const wxRect& rect, int flags = 0 );
+
+    virtual wxSize GetCheckBoxSize(wxWindow *win, int flags = 0);
+
+    virtual wxSize GetCheckMarkSize(wxWindow *win);
+
+    virtual wxSize GetExpanderSize(wxWindow* win);
 
     virtual void DrawPushButton(wxWindow *win, wxDC& dc,
                                 const wxRect& rect, int flags = 0 );
@@ -382,7 +389,7 @@ public:
     */
     virtual int DrawHeaderButton(wxWindow* win, wxDC& dc, const wxRect& rect,
                                  int flags = 0,
-                                 wxHeaderSortIconType sortArrow = wxHDR_SORT_ICON_NONE, wxHeaderButtonParams* params = NULL) = 0;
+                                 wxHeaderSortIconType sortArrow = wxHDR_SORT_ICON_NONE, wxHeaderButtonParams* params = nullptr) = 0;
 
     /**
         Draw the contents of a header control button (label, sort arrows,
@@ -397,7 +404,7 @@ public:
     */
     virtual int DrawHeaderButtonContents(wxWindow* win, wxDC& dc,
                                          const wxRect& rect, int flags = 0,
-                                         wxHeaderSortIconType sortArrow = wxHDR_SORT_ICON_NONE, wxHeaderButtonParams* params = NULL) = 0;
+                                         wxHeaderSortIconType sortArrow = wxHDR_SORT_ICON_NONE, wxHeaderButtonParams* params = nullptr) = 0;
 
     /**
         Draw a selection rectangle underneath the text as used e.g. in a
@@ -462,9 +469,12 @@ public:
     /**
         Returns the size of a collapse button.
 
+        @a dc parameter was of type wxDC until wxWidgets 3.3.0, which changed
+        it to wxReadOnlyDC as this function doesn't modify the DC contents.
+
         @since 3.1.0
     */
-    virtual wxSize GetCollapseButtonSize(wxWindow *win, wxDC& dc) = 0;
+    virtual wxSize GetCollapseButtonSize(wxWindow *win, wxReadOnlyDC& dc) = 0;
 
     /**
         Draw the border for sash window: this border must be such that the sash
@@ -513,7 +523,7 @@ public:
     /**
         Draw a title bar button in the given state.
 
-        This function is currently only available under MSW and OS X (and only
+        This function is currently only available under MSW and macOS (and only
         for wxTITLEBAR_BUTTON_CLOSE under the latter), its best replacement for
         the other platforms is to use wxArtProvider to retrieve the bitmaps for
         @c wxART_HELP and @c wxART_CLOSE (but not any other title bar buttons
@@ -530,6 +540,17 @@ public:
                                     wxTitleBarButton button,
                                     int flags = 0) = 0;
 
+    /**
+        Draw a check mark.
+
+        @a flags may have the @c wxCONTROL_DISABLED bit set, see
+        @ref wxCONTROL_FLAGS.
+
+        @since 3.1.3
+    */
+    virtual void DrawCheckMark(wxWindow* win, wxDC& dc, const wxRect& rect,
+                               int flags = 0) = 0;
+
     /**
         Return the currently used renderer.
     */
@@ -555,8 +576,33 @@ public:
 
         @param win A valid, i.e. non-null, window pointer which is used to get
             the theme defining the checkbox size under some platforms.
+
+        @param flags The only acceptable flag is @c wxCONTROL_CELL which means
+            that just the size of the checkbox itself is returned, without any
+            margins that are included by default. This parameter is only
+            available in wxWidgets 3.1.4 or later.
     */
-    virtual wxSize GetCheckBoxSize(wxWindow* win) = 0;
+    virtual wxSize GetCheckBoxSize(wxWindow* win, int flags = 0) = 0;
+
+    /**
+        Returns the size of a check mark.
+
+        @param win A valid, i.e. non-null, window pointer which is used to get
+            the theme defining the checkmark size under some platforms.
+
+        @since 3.1.3
+    */
+    virtual wxSize GetCheckMarkSize(wxWindow* win) = 0;
+
+    /**
+        Returns the size of the expander used in tree-like controls.
+
+        @param win A valid, i.e. non-null, window pointer which is used to get
+            the theme defining the expander size under some platforms.
+
+        @since 3.1.3
+     */
+    virtual wxSize GetExpanderSize(wxWindow* win) = 0;
 
     /**
         Returns the height of a header button, either a fixed platform height if
@@ -635,7 +681,7 @@ public:
 struct wxRendererVersion
 {
     wxRendererVersion(int version_, int age_);
-    
+
     /**
         Checks if the main program is compatible with the renderer having the version
         @e ver, returns @true if it is and @false otherwise.
