@@ -29,14 +29,14 @@ public:
     */
     wxUniChar();
 
-    //@{
+    ///@{
     /**
         Create a character from the 8-bit character value @a c using the
         current locale encoding.
     */
     wxUniChar(char c);
     wxUniChar(unsigned char c);
-    //@}
+    ///@}
 
     wxUniChar(int c);
     wxUniChar(unsigned int c);
@@ -72,7 +72,7 @@ public:
 
         @param c
             An output pointer to the value of this Unicode character as a @c
-            char. Must be non-@NULL.
+            char. Must be non-null.
         @return
             @true if the object is an 8 bit char and @a c was filled with its
             value as char or @false otherwise (@a c won't be modified then).
@@ -83,7 +83,83 @@ public:
      */
     bool GetAsChar(char *c) const;
 
-    //@{
+    /**
+        Returns true if the character is a BMP character (i.e.\ if its value is less than 0x10000).
+
+        @since 3.1.1
+    */
+    bool IsBMP() const;
+
+    /**
+        Returns true if the character is a BMP character (i.e.\ if its value is less than 0x10000).
+
+        @param value
+            The Unicode code point of the character.
+
+        @since 3.1.1
+    */
+    static bool IsBMP(wxUint32 value);
+
+    /**
+        Returns true if the character is a supplementary character (i.e.\ between 0x10000 and 0x10FFFF).
+
+        @since 3.1.1
+    */
+    bool IsSupplementary() const;
+
+    /**
+        Returns true if the character is a supplementary character (i.e.\ between 0x10000 and 0x10FFFF).
+
+        @param value
+            The Unicode code point of the character.
+
+        @since 3.1.1
+    */
+    static bool IsSupplementary(wxUint32 value);
+
+    /**
+        Returns the high surrogate code unit for the supplementary character.
+
+        @pre IsSupplementary() const
+
+        @since 3.1.1
+    */
+    wxUint16 HighSurrogate() const;
+
+    /**
+        Returns the high surrogate code unit for the supplementary character.
+
+        @param value
+            The Unicode code point of the character.
+
+        @pre IsSupplementary(wxUint32 value)
+
+        @since 3.1.1
+    */
+    static wxUint16 HighSurrogate(wxUint32 value);
+
+    /**
+        Returns the low surrogate code unit for the supplementary character.
+
+        @pre IsSupplementary() const
+
+        @since 3.1.1
+    */
+    wxUint16 LowSurrogate() const;
+
+    /**
+        Returns the low surrogate code unit for the supplementary character.
+
+        @param value
+            The Unicode code point of the character.
+
+        @pre IsSupplementary(wxUint32 value)
+
+        @since 3.1.1
+    */
+    static wxUint16 LowSurrogate(wxUint32 value);
+
+    ///@{
     /**
         Conversions to char and wchar_t types: all of those are needed to be
         able to pass wxUniChars to various standard narrow and wide character
@@ -100,9 +176,9 @@ public:
     operator unsigned short int() const;
     operator wxLongLong_t() const;
     operator wxULongLong_t() const;
-    //@}
+    ///@}
 
-    //@{
+    ///@{
     /**
         Assignment operators
     */
@@ -119,7 +195,7 @@ public:
     wxUniChar& operator=(unsigned short int c);
     wxUniChar& operator=(wxLongLong_t c);
     wxUniChar& operator=(wxULongLong_t c);
-    //@}
+    ///@}
 };
 
 
