@@ -2,7 +2,6 @@
 // Name:        src/msw/msgdlg.cpp
 // Purpose:     wxMessageDialog
 // Author:      Julian Smart
-// Modified by:
 // Created:     04/01/98
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
@@ -11,9 +10,6 @@
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #if wxUSE_MSGDLG
 
@@ -23,18 +19,19 @@
     #include "wx/intl.h"
     #include "wx/utils.h"
     #include "wx/msw/private.h"
-    #include "wx/hashmap.h"
 #endif
 
 #include "wx/ptr_scpd.h"
 #include "wx/dynlib.h"
 #include "wx/msw/private/button.h"
+#include "wx/msw/private/darkmode.h"
 #include "wx/msw/private/metrics.h"
 #include "wx/msw/private/msgdlg.h"
 #include "wx/modalhook.h"
 #include "wx/fontutil.h"
 #include "wx/textbuf.h"
 #include "wx/display.h"
+#include "wx/translation.h"
 
 // Interestingly, this symbol currently seems to be absent from Platform SDK
 // headers but it is documented at MSDN.
@@ -46,14 +43,6 @@ using namespace wxMSWMessageDialog;
 
 wxIMPLEMENT_CLASS(wxMessageDialog, wxDialog);
 
-// there can potentially be one message box per thread so we use a hash map
-// with thread ids as keys and (currently shown) message boxes as values
-//
-// TODO: replace this with wxTLS once it's available
-WX_DECLARE_HASH_MAP(unsigned long, wxMessageDialog *,
-                    wxIntegerHash, wxIntegerEqual,
-                    wxMessageDialogMap);
-
 // the order in this array is the one in which buttons appear in the
 // message box
 const wxMessageDialog::ButtonAccessors wxMessageDialog::ms_buttons[] =
@@ -67,12 +56,9 @@ const wxMessageDialog::ButtonAccessors wxMessageDialog::ms_buttons[] =
 namespace
 {
 
-wxMessageDialogMap& HookMap()
-{
-    static wxMessageDialogMap s_Map;
-
-    return s_Map;
-}
+// Different threads could potentially show message boxes at the same time, so
+// remember the window showing it in a thread-specific variable.
+thread_local wxMessageDialog* gs_currentDialog = nullptr;
 
 /*
     All this code is used for adjusting the message box layout when we mess
@@ -88,10 +74,7 @@ wxMessageDialogMap& HookMap()
 void ScreenRectToClient(HWND hwnd, RECT& rc)
 {
     // map from desktop (i.e. screen) coordinates to ones of this window
-    //
-    // notice that a RECT is laid out as 2 consecutive POINTs so the cast is
-    // valid
-    ::MapWindowPoints(HWND_DESKTOP, hwnd, reinterpret_cast<POINT *>(&rc), 2);
+    wxMapWindowPoints(HWND_DESKTOP, hwnd, &rc);
 }
 
 // set window position to the given rect
@@ -118,13 +101,9 @@ void MoveWindowToScreenRect(HWND hwnd, RECT rc)
 WXLRESULT wxCALLBACK
 wxMessageDialog::HookFunction(int code, WXWPARAM wParam, WXLPARAM lParam)
 {
-    // Find the thread-local instance of wxMessageDialog
-    const DWORD tid = ::GetCurrentThreadId();
-    wxMessageDialogMap::iterator node = HookMap().find(tid);
-    wxCHECK_MSG( node != HookMap().end(), false,
-                    wxT("bogus thread id in wxMessageDialog::Hook") );
-
-    wxMessageDialog *  const wnd = node->second;
+    // Get the thread-local instance of wxMessageDialog
+    wxMessageDialog *  const wnd = gs_currentDialog;
+    wxCHECK_MSG( wnd, false, "No valid wxMessageDialog?" );
 
     const HHOOK hhook = (HHOOK)wnd->m_hook;
     const LRESULT rc = ::CallNextHookEx(hhook, code, wParam, lParam);
@@ -133,10 +112,10 @@ wxMessageDialog::HookFunction(int code, WXWPARAM wParam, WXLPARAM lParam)
     {
         // we won't need this hook any longer
         ::UnhookWindowsHookEx(hhook);
-        wnd->m_hook = NULL;
-        HookMap().erase(tid);
+        wnd->m_hook = nullptr;
+        gs_currentDialog = nullptr;
 
-        wnd->SetHWND((HWND)wParam);
+        TempHWNDSetter set(wnd, (WXHWND)wParam);
 
         // replace the static text with an edit control if the message box is
         // too big to fit the display
@@ -151,9 +130,6 @@ wxMessageDialog::HookFunction(int code, WXWPARAM wParam, WXLPARAM lParam)
         if ( wnd->GetMessageDialogStyle() & wxCENTER )
             wnd->Center(); // center on parent
         //else: default behaviour, center on screen
-
-        // there seems to be no reason to leave it set
-        wnd->SetHWND(NULL);
     }
 
     return rc;
@@ -174,9 +150,9 @@ void wxMessageDialog::ReplaceStaticWithEdit()
     // find the static control to replace: normally there are two of them, the
     // icon and the text itself so search for all of them and ignore the icon
     // ones
-    HWND hwndStatic = ::FindWindowEx(GetHwnd(), NULL, wxT("STATIC"), NULL);
+    HWND hwndStatic = ::FindWindowEx(GetHwnd(), nullptr, wxT("STATIC"), nullptr);
     if ( ::GetWindowLong(hwndStatic, GWL_STYLE) & SS_ICON )
-        hwndStatic = ::FindWindowEx(GetHwnd(), hwndStatic, wxT("STATIC"), NULL);
+        hwndStatic = ::FindWindowEx(GetHwnd(), hwndStatic, wxT("STATIC"), nullptr);
 
     if ( !hwndStatic )
     {
@@ -196,8 +172,8 @@ void wxMessageDialog::ReplaceStaticWithEdit()
     // some space above and below it
     const int hText = (7*rectDisplay.height)/8 -
                       (
-                         2*::GetSystemMetrics(SM_CYFIXEDFRAME) +
-                         ::GetSystemMetrics(SM_CYCAPTION) +
+                         2*wxGetSystemMetrics(SM_CYFIXEDFRAME, this) +
+                         wxGetSystemMetrics(SM_CYCAPTION, this) +
                          5*GetCharHeight() // buttons + margins
                       );
     const int dh = (rc.bottom - rc.top) - hText; // vertical space we save
@@ -210,8 +186,8 @@ void wxMessageDialog::ReplaceStaticWithEdit()
     // NB: you would have thought that 2*SM_CXEDGE would be enough but it
     //     isn't, somehow, and the text control breaks lines differently from
     //     the static one so fudge by adding some extra space
-    const int dw = ::GetSystemMetrics(SM_CXVSCROLL) +
-                        4*::GetSystemMetrics(SM_CXEDGE);
+    const int dw = wxGetSystemMetrics(SM_CXVSCROLL, this) +
+                        4*wxGetSystemMetrics(SM_CXEDGE, this);
     rc.right += dw;
 
 
@@ -241,9 +217,9 @@ void wxMessageDialog::ReplaceStaticWithEdit()
                         rc.left, rc.top,
                         rc.right - rc.left, rc.bottom - rc.top,
                         GetHwnd(),
-                        NULL,
+                        nullptr,
                         wxGetInstance(),
-                        NULL
+                        nullptr
                       );
 
     if ( !hwndEdit )
@@ -399,8 +375,11 @@ void wxMessageDialog::AdjustButtonLabels()
 /* static */
 wxFont wxMessageDialog::GetMessageFont()
 {
-    const NONCLIENTMETRICS& ncm = wxMSWImpl::GetNonClientMetrics();
-    return wxNativeFontInfo(ncm.lfMessageFont);
+    const wxWindow* win = wxApp::GetMainTopWindow();
+    const wxNativeFontInfo
+        info(wxMSWImpl::GetNonClientMetrics(win).lfMessageFont, win);
+
+    return info;
 }
 
 int wxMessageDialog::ShowMessageBox()
@@ -417,7 +396,7 @@ int wxMessageDialog::ShowMessageBox()
 
     // use the top level window as parent if none specified
     m_parent = GetParentForModalDialog();
-    HWND hWnd = m_parent ? GetHwndOf(m_parent) : NULL;
+    HWND hWnd = m_parent ? GetHwndOf(m_parent) : nullptr;
 
 #if wxUSE_INTL
     // native message box always uses the current user locale but the program
@@ -426,8 +405,8 @@ int wxMessageDialog::ShowMessageBox()
     // way to translate them and so we must assume they were already
     // translated) to avoid mismatch between the language of the message box
     // text and its buttons
-    wxLocale * const loc = wxGetLocale();
-    if ( loc && loc->GetLanguage() != wxLocale::GetSystemLanguage() )
+    wxTranslations* currentTranslations = wxTranslations::Get();
+    if ( currentTranslations )
     {
         if ( m_dialogStyle & wxYES_NO &&
                 (GetCustomYesLabel().empty() && GetCustomNoLabel().empty()) )
@@ -523,8 +502,8 @@ int wxMessageDialog::ShowMessageBox()
     // control with an edit one)
     const DWORD tid = ::GetCurrentThreadId();
     m_hook = ::SetWindowsHookEx(WH_CBT,
-                                &wxMessageDialog::HookFunction, NULL, tid);
-    HookMap()[tid] = this;
+                                &wxMessageDialog::HookFunction, nullptr, tid);
+    gs_currentDialog = this;
 
     // do show the dialog
     const int msAns = MessageBox
@@ -542,18 +521,16 @@ int wxMessageDialog::ShowModal()
 {
     WX_HOOK_MODAL_DIALOG();
 
-#ifdef wxHAS_MSW_TASKDIALOG
-    if ( HasNativeTaskDialog() )
-    {
-        TaskDialogIndirect_t taskDialogIndirect = GetTaskDialogIndirectFunc();
-        wxCHECK_MSG( taskDialogIndirect, wxID_CANCEL, wxS("no task dialog?") );
+    wxWindowDisabler disableOthers(this, GetParentForModalDialog());
 
+    if ( TaskDialogIndirect_t taskDialogIndirect = GetTaskDialogIndirectFunc() )
+    {
         WinStruct<TASKDIALOGCONFIG> tdc;
         wxMSWTaskDialogConfig wxTdc( *this );
         wxTdc.MSWCommonTaskDialogInit( tdc );
 
         int msAns;
-        HRESULT hr = taskDialogIndirect( &tdc, &msAns, NULL, NULL );
+        HRESULT hr = taskDialogIndirect( &tdc, &msAns, nullptr, nullptr );
         if ( FAILED(hr) )
         {
             wxLogApiError( "TaskDialogIndirect", hr );
@@ -572,7 +549,6 @@ int wxMessageDialog::ShowModal()
 
         return MSWTranslateReturnCode( msAns );
     }
-#endif // wxHAS_MSW_TASKDIALOG
 
     return ShowMessageBox();
 }
@@ -591,13 +567,11 @@ long wxMessageDialog::GetEffectiveIcon() const
 
 void wxMessageDialog::DoCentre(int dir)
 {
-#ifdef wxHAS_MSW_TASKDIALOG
     // Task dialog is always centered on its parent window and trying to center
     // it manually doesn't work because its HWND is not created yet so don't
     // even try as this would only result in (debug) error messages.
     if ( HasNativeTaskDialog() )
         return;
-#endif // wxHAS_MSW_TASKDIALOG
 
     wxMessageDialogBase::DoCentre(dir);
 }
@@ -606,7 +580,23 @@ void wxMessageDialog::DoCentre(int dir)
 // Helpers of the wxMSWMessageDialog namespace
 // ----------------------------------------------------------------------------
 
-#ifdef wxHAS_MSW_TASKDIALOG
+namespace
+{
+
+HRESULT CALLBACK
+wxTaskDialogCallback(HWND hwnd, UINT msg, WPARAM, LPARAM, LONG_PTR)
+{
+    switch ( msg )
+    {
+        case TDN_DIALOG_CONSTRUCTED:
+            wxMSWDarkMode::EnableForTLW(hwnd);
+            break;
+    }
+
+    return S_OK;
+}
+
+} // anonymous namespace
 
 wxMSWTaskDialogConfig::wxMSWTaskDialogConfig(const wxMessageDialogBase& dlg)
                      : buttons(new TASKDIALOG_BUTTON[MAX_BUTTONS])
@@ -662,7 +652,7 @@ void wxMSWTaskDialogConfig::MSWCommonTaskDialogInit(TASKDIALOGCONFIG &tdc)
     tdc.pszWindowTitle = caption.t_str();
 
     // use the top level window as parent if none specified
-    tdc.hwndParent = parent ? GetHwndOf(parent) : NULL;
+    tdc.hwndParent = parent ? GetHwndOf(parent) : nullptr;
 
     if ( wxApp::MSWGetDefaultLayout(parent) == wxLayout_RightToLeft )
         tdc.dwFlags |= TDF_RTL_LAYOUT;
@@ -756,6 +746,8 @@ void wxMSWTaskDialogConfig::MSWCommonTaskDialogInit(TASKDIALOGCONFIG &tdc)
 
         AddTaskDialogButton(tdc, IDHELP, 0 /* not used */, btnHelpLabel);
     }
+
+    tdc.pfCallback = wxTaskDialogCallback;
 }
 
 void wxMSWTaskDialogConfig::AddTaskDialogButton(TASKDIALOGCONFIG &tdc,
@@ -789,7 +781,7 @@ wxCRIT_SECT_DECLARE(gs_csTaskDialogIndirect);
 
 TaskDialogIndirect_t wxMSWMessageDialog::GetTaskDialogIndirectFunc()
 {
-    // Initialize the function pointer to an invalid value different from NULL
+    // Initialize the function pointer to an invalid value different from nullptr
     // to avoid reloading comctl32.dll and trying to resolve it every time
     // we're called if task dialog is not available (notice that this may
     // happen even under Vista+ if we don't use comctl32.dll v6).
@@ -808,19 +800,9 @@ TaskDialogIndirect_t wxMSWMessageDialog::GetTaskDialogIndirectFunc()
     return s_TaskDialogIndirect;
 }
 
-#endif // wxHAS_MSW_TASKDIALOG
-
 bool wxMSWMessageDialog::HasNativeTaskDialog()
 {
-#ifdef wxHAS_MSW_TASKDIALOG
-    if ( wxGetWinVersion() >= wxWinVersion_6 )
-    {
-        if ( wxMSWMessageDialog::GetTaskDialogIndirectFunc() )
-            return true;
-    }
-#endif // wxHAS_MSW_TASKDIALOG
-
-    return false;
+    return wxMSWMessageDialog::GetTaskDialogIndirectFunc() != nullptr;
 }
 
 int wxMSWMessageDialog::MSWTranslateReturnCode(int msAns)
