@@ -1,11 +1,9 @@
 ///////////////////////////////////////////////////////////////////////////////
-// Name:        msw/snglinst.cpp
+// Name:        src/msw/snglinst.cpp
 // Purpose:     implements wxSingleInstanceChecker class for Win32 using
 //              named mutexes
 // Author:      Vadim Zeitlin
-// Modified by:
 // Created:     08.06.01
-// RCS-ID:      $Id$
 // Copyright:   (c) 2001 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -21,11 +19,8 @@
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
-#if wxUSE_SNGLINST_CHECKER && defined(__WIN32__)
+#if wxUSE_SNGLINST_CHECKER
 
 #ifndef WX_PRECOMP
     #include "wx/string.h"
@@ -47,12 +42,12 @@ public:
     {
         // we don't care about m_wasOpened, it can't be accessed before being
         // initialized
-        m_hMutex = NULL;
+        m_hMutex = nullptr;
     }
 
     bool Create(const wxString& name)
     {
-        m_hMutex = ::CreateMutex(NULL, FALSE, name.t_str());
+        m_hMutex = ::CreateMutex(nullptr, FALSE, name.t_str());
         if ( !m_hMutex )
         {
             wxLogLastError(wxT("CreateMutex"));
@@ -89,7 +84,7 @@ private:
     // the result of the CreateMutex() call
     bool m_wasOpened;
 
-    // the mutex handle, may be NULL
+    // the mutex handle, may be null
     HANDLE m_hMutex;
 
     wxDECLARE_NO_COPY_CLASS(wxSingleInstanceCheckerImpl);
