@@ -2,9 +2,7 @@
 // Name:        src/univ/themes/mono.cpp
 // Purpose:     wxUniversal theme for monochrome displays
 // Author:      Vadim Zeitlin
-// Modified by:
 // Created:     2006-08-27
-// RCS-ID:      $Id: mono.cpp,v 1.18 2006/12/24 19:06:38 VS Exp $
 // Copyright:   (c) 2006 REA Elektronik GmbH
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -20,15 +18,13 @@
 // for compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #include "wx/univ/theme.h"
 
 #if wxUSE_THEME_MONO
 
 #ifndef WX_PRECOMP
+    #include "wx/window.h"
     #include "wx/dc.h"
 #endif // WX_PRECOMP
 
@@ -53,6 +49,13 @@ class wxMonoRenderer : public wxStdRenderer
 public:
     wxMonoRenderer(const wxColourScheme *scheme);
 
+    virtual void DrawLabel(wxDC& dc,
+                           const wxString& label,
+                           const wxRect& rect,
+                           int flags = 0,
+                           int alignment = wxALIGN_LEFT | wxALIGN_TOP,
+                           int indexAccel = -1,
+                           wxRect *rectBounds = nullptr);
     virtual void DrawButtonLabel(wxDC& dc,
                                  const wxString& label,
                                  const wxBitmap& image,
@@ -60,14 +63,14 @@ public:
                                  int flags = 0,
                                  int alignment = wxALIGN_LEFT | wxALIGN_TOP,
                                  int indexAccel = -1,
-                                 wxRect *rectBounds = NULL);
+                                 wxRect *rectBounds = nullptr);
 
-    virtual void DrawFocusRect(wxDC& dc, const wxRect& rect, int flags = 0);
+    virtual void DrawFocusRect(wxWindow* win, wxDC& dc, const wxRect& rect, int flags = 0);
 
     virtual void DrawButtonBorder(wxDC& dc,
                                   const wxRect& rect,
                                   int flags = 0,
-                                  wxRect *rectIn = NULL);
+                                  wxRect *rectIn = nullptr);
 
     virtual void DrawHorizontalLine(wxDC& dc,
                                     wxCoord y, wxCoord x1, wxCoord x2);
@@ -115,7 +118,7 @@ public:
                                  wxOrientation orient,
                                  int flags = 0,
                                  long style = 0,
-                                 wxRect *rectShaft = NULL);
+                                 wxRect *rectShaft = nullptr);
 
     virtual void DrawSliderThumb(wxDC& dc,
                                  const wxRect& rect,
@@ -514,9 +517,9 @@ WX_IMPLEMENT_THEME(wxMonoTheme, mono, wxTRANSLATE("Simple monochrome theme"));
 
 wxMonoTheme::wxMonoTheme()
 {
-    m_scheme = NULL;
-    m_renderer = NULL;
-    m_artProvider = NULL;
+    m_scheme = nullptr;
+    m_renderer = nullptr;
+    m_artProvider = nullptr;
 }
 
 wxMonoTheme::~wxMonoTheme()
@@ -560,7 +563,7 @@ wxInputHandler *wxMonoTheme::GetInputHandler(const wxString& WXUNUSED(control),
                                              wxInputConsumer *consumer)
 {
     // no special input handlers so far
-    return consumer->DoGetStdInputHandler(NULL);
+    return consumer->DoGetStdInputHandler(nullptr);
 }
 
 // ============================================================================
@@ -577,7 +580,7 @@ wxColour wxMonoColourScheme::GetBackground(wxWindow *win) const
     }
 
     // doesn't depend on the state
-    if ( !col.Ok() )
+    if ( !col.IsOk() )
     {
         col = GetBg();
     }
@@ -605,7 +608,7 @@ wxColour wxMonoColourScheme::Get(wxMonoColourScheme::StdColour col) const
 
         case MAX:
         default:
-            wxFAIL_MSG(_T("invalid standard colour"));
+            wxFAIL_MSG(wxT("invalid standard colour"));
             // fall through
 
         case SHADOW_DARK:
@@ -650,15 +653,16 @@ wxRect wxMonoRenderer::GetBorderDimensions(wxBorder border) const
         case wxBORDER_STATIC:
         case wxBORDER_RAISED:
         case wxBORDER_SUNKEN:
+        case wxBORDER_THEME:
             width = 1;
             break;
-
+  /*
         case wxBORDER_DOUBLE:
             width = 2;
             break;
-
+   */
         default:
-            wxFAIL_MSG(_T("unknown border type"));
+            wxFAIL_MSG(wxT("unknown border type"));
             // fall through
 
         case wxBORDER_DEFAULT:
@@ -702,7 +706,7 @@ wxMonoRenderer::DrawVerticalLine(wxDC& dc, wxCoord x, wxCoord y1, wxCoord y2)
     dc.DrawLine(x, y1, x, y2 + 1);
 }
 
-void wxMonoRenderer::DrawFocusRect(wxDC& dc, const wxRect& rect, int flags)
+void wxMonoRenderer::DrawFocusRect(wxWindow* WXUNUSED(win), wxDC& dc, const wxRect& rect, int flags)
 {
     // no need to draw the focus rect for selected items, it would be invisible
     // anyhow
@@ -718,6 +722,17 @@ void wxMonoRenderer::DrawFocusRect(wxDC& dc, const wxRect& rect, int flags)
 // label
 // ----------------------------------------------------------------------------
 
+void wxMonoRenderer::DrawLabel(wxDC& dc,
+                               const wxString& label,
+                               const wxRect& rect,
+                               int WXUNUSED(flags),
+                               int alignment,
+                               int indexAccel,
+                               wxRect *rectBounds)
+{
+    dc.DrawLabel(label, wxNullBitmap, rect, alignment, indexAccel, rectBounds);
+}
+
 void wxMonoRenderer::DrawButtonLabel(wxDC& dc,
                                      const wxString& label,
                                      const wxBitmap& image,
@@ -750,7 +765,7 @@ wxBitmap wxMonoRenderer::GetIndicator(IndicatorType indType, int flags)
     GetIndicatorsFromFlags(flags, indState, indStatus);
 
     wxBitmap& bmp = m_bmpIndicators[indType][indState][indStatus];
-    if ( !bmp.Ok() )
+    if ( !bmp.IsOk() )
     {
         const char **xpm = ms_xpmIndicators[indType][indState][indStatus];
         if ( xpm )
@@ -767,7 +782,7 @@ wxBitmap wxMonoRenderer::GetFrameButtonBitmap(FrameButtonType type)
 {
     if ( type == FrameButton_Close )
     {
-        if ( !m_bmpFrameClose.Ok() )
+        if ( !m_bmpFrameClose.IsOk() )
         {
             static const char *xpmFrameClose[] = {
             /* columns rows colors chars-per-pixel */
@@ -809,19 +824,19 @@ void wxMonoRenderer::DrawToolBarButton(wxDC& WXUNUSED(dc),
                                        long WXUNUSED(style),
                                        int WXUNUSED(tbarStyle))
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 }
 
 wxSize wxMonoRenderer::GetToolBarButtonSize(wxCoord *WXUNUSED(separator)) const
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 
     return wxSize();
 }
 
 wxSize wxMonoRenderer::GetToolBarMargin() const
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 
     return wxSize();
 }
@@ -842,19 +857,19 @@ void wxMonoRenderer::DrawTab(wxDC& WXUNUSED(dc),
                              int WXUNUSED(flags),
                              int WXUNUSED(indexAccel))
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 }
 
 wxSize wxMonoRenderer::GetTabIndent() const
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 
     return wxSize();
 }
 
 wxSize wxMonoRenderer::GetTabPadding() const
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 
     return wxSize();
 }
@@ -872,7 +887,7 @@ void wxMonoRenderer::GetComboBitmaps(wxBitmap *WXUNUSED(bmpNormal),
                                      wxBitmap *WXUNUSED(bmpPressed),
                                      wxBitmap *WXUNUSED(bmpDisabled))
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 }
 
 #endif // wxUSE_COMBOBOX
@@ -889,7 +904,7 @@ void wxMonoRenderer::DrawMenuBarItem(wxDC& WXUNUSED(dc),
                                      int WXUNUSED(flags),
                                      int WXUNUSED(indexAccel))
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 }
 
 void wxMonoRenderer::DrawMenuItem(wxDC& WXUNUSED(dc),
@@ -901,19 +916,19 @@ void wxMonoRenderer::DrawMenuItem(wxDC& WXUNUSED(dc),
                                   int WXUNUSED(flags),
                                   int WXUNUSED(indexAccel))
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 }
 
 void wxMonoRenderer::DrawMenuSeparator(wxDC& WXUNUSED(dc),
                                        wxCoord WXUNUSED(y),
                                        const wxMenuGeometryInfo& WXUNUSED(geomInfo))
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 }
 
 wxSize wxMonoRenderer::GetMenuBarItemSize(const wxSize& WXUNUSED(sizeText)) const
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 
     return wxSize();
 }
@@ -921,9 +936,9 @@ wxSize wxMonoRenderer::GetMenuBarItemSize(const wxSize& WXUNUSED(sizeText)) cons
 wxMenuGeometryInfo *wxMonoRenderer::GetMenuGeometry(wxWindow *WXUNUSED(win),
                                                     const wxMenu& WXUNUSED(menu)) const
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 
-    return NULL;
+    return nullptr;
 }
 
 #endif // wxUSE_MENUS
@@ -942,7 +957,7 @@ void wxMonoRenderer::DrawSliderShaft(wxDC& WXUNUSED(dc),
                                      long WXUNUSED(style),
                                      wxRect *WXUNUSED(rectShaft))
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 }
 
 
@@ -952,7 +967,7 @@ void wxMonoRenderer::DrawSliderThumb(wxDC& WXUNUSED(dc),
                                      int WXUNUSED(flags),
                                      long WXUNUSED(style))
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 }
 
 void wxMonoRenderer::DrawSliderTicks(wxDC& WXUNUSED(dc),
@@ -965,19 +980,19 @@ void wxMonoRenderer::DrawSliderTicks(wxDC& WXUNUSED(dc),
                                      int WXUNUSED(flags),
                                      long WXUNUSED(style))
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 }
 
 wxCoord wxMonoRenderer::GetSliderDim() const
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 
     return 0;
 }
 
 wxCoord wxMonoRenderer::GetSliderTickLen() const
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 
     return 0;
 }
@@ -988,7 +1003,7 @@ wxRect wxMonoRenderer::GetSliderShaftRect(const wxRect& WXUNUSED(rect),
                                           wxOrientation WXUNUSED(orient),
                                           long WXUNUSED(style)) const
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 
     return wxRect();
 }
@@ -997,7 +1012,7 @@ wxSize wxMonoRenderer::GetSliderThumbSize(const wxRect& WXUNUSED(rect),
                                           int WXUNUSED(lenThumb),
                                           wxOrientation WXUNUSED(orient)) const
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 
     return wxSize();
 }
@@ -1006,7 +1021,7 @@ wxSize wxMonoRenderer::GetSliderThumbSize(const wxRect& WXUNUSED(rect),
 
 wxSize wxMonoRenderer::GetProgressBarStep() const
 {
-    wxFAIL_MSG(_T("TODO"));
+    wxFAIL_MSG(wxT("TODO"));
 
     return wxSize();
 }
@@ -1022,10 +1037,10 @@ void wxMonoRenderer::DrawArrow(wxDC& dc,
                                int WXUNUSED(flags))
 {
     ArrowDirection arrowDir = GetArrowDirection(dir);
-    wxCHECK_RET( arrowDir != Arrow_Max, _T("invalid arrow direction") );
+    wxCHECK_RET( arrowDir != Arrow_Max, wxT("invalid arrow direction") );
 
     wxBitmap& bmp = m_bmpArrows[arrowDir];
-    if ( !bmp.Ok() )
+    if ( !bmp.IsOk() )
     {
         bmp = wxBitmap(ms_xpmArrows[arrowDir]);
     }
@@ -1041,7 +1056,7 @@ void wxMonoRenderer::DrawScrollbarThumb(wxDC& dc,
 {
     DrawSolidRect(dc, wxMONO_BG_COL, rect);
 
-    // manually draw stipple pattern (wxDFB doesn't implement the wxSTIPPLE
+    // manually draw stipple pattern (wxDFB doesn't implement the wxBRUSHSTYLE_STIPPLE
     // brush style):
     dc.SetPen(m_penFg);
     for ( wxCoord y = rect.GetTop(); y <= rect.GetBottom(); y++ )
