@@ -2,9 +2,7 @@
 // Name:        helpview.h
 // Purpose:     HelpView application class
 // Author:      Vaclav Slavik, Julian Smart
-// Modified by:
 // Created:     2002-07-09
-// RCS-ID:      $Id$
 // Copyright:   (c) 2002 Vaclav Slavik, Julian Smart and others
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -36,7 +34,7 @@ public:
 
 #ifdef __WXMAC__
     /// Respond to Apple Event for opening a document
-    virtual void MacOpenFile(const wxString& filename);
+    virtual void MacOpenFiles(const wxArrayString& fileNames);
 #endif
 
     /// Prompt the user for a book to open
